@@ -4,7 +4,7 @@
 //! They range from the generic "4.04
 //! Not Found" responder up to a handler that creates a [write] formatter for GET-only resources,
 //! and even provides [block-wise transfer] that.
-//! The [SimpleCBORWrapper] enables the easy creation of [serde_cbor] based
+//! The [SimpleWrapper] enables the easy creation of [serde_cbor] based
 //! resource implementations with GET, PUT and POST support in CBOR format.
 //! The `HandlerBuilder` implements crude static path based routing
 //! that may suffice for some applications, and is also useful to get started quickly.
@@ -45,6 +45,7 @@
 )]
 
 use coap_message::{MessageOption, MutableWritableMessage, ReadableMessage};
+use core::marker::PhantomData;
 
 pub mod helpers;
 pub mod option_processing;
@@ -280,21 +281,21 @@ where
     // (After that, could be that it's warranted as it's now more than a few 4-liners, could be
     // that not).
 
-    type Record<'b>
+    type Record<'b> = ForkingRecord<PrefixedRecord<'b, H1::Record<'b>>, H2::Record<'b>>
     where
         Self: 'b,
-    = ForkingRecord<PrefixedRecord<'b, H1::Record<'b>>, H2::Record<'b>>;
+    ;
 
-    type Reporter<'b>
-    where
-        Self: 'b,
-    = core::iter::Chain<
+    type Reporter<'b> = core::iter::Chain<
         core::iter::Map<H2::Reporter<'b>, fn(H2::Record<'b>) -> Self::Record<'b>>,
         core::iter::Map<
             core::iter::Zip<H1::Reporter<'b>, core::iter::Repeat<&'b [&'b str]>>,
             fn((H1::Record<'b>, &'b [&'b str])) -> Self::Record<'b>,
         >,
-    >;
+    >
+    where
+        Self: 'b,
+    ;
     fn report(&self) -> Self::Reporter<'_> {
         fn first<'c, H1R, H2R>(
             prefixed_prefix: (H1R, &'c [&'c str]),
@@ -371,18 +372,19 @@ where
     H1: Handler<RequestData = RD1> + wkc::Reporting,
     H2: Handler<RequestData = RD2> + wkc::Reporting,
 {
-    type Record<'b>
+    type Record<'b> = ForkingRecord<PrefixedRecord<'b, H1::Record<'b>>, H2::Record<'b>>
     where
         Self: 'b,
-    = ForkingRecord<PrefixedRecord<'b, H1::Record<'b>>, H2::Record<'b>>;
+    ;
 
     // FIXME: one of these is copied over to wkc::Reporting for ForkingHandler, see there.
 
     #[cfg(feature = "nontrivial_option_processing")]
-    type Reporter<'b>
+    type Reporter<'b> = impl Iterator<Item = Self::Record<'b>>
     where
         Self: 'b,
-    = impl Iterator<Item = Self::Record<'b>>;
+    ;
+
     #[cfg(feature = "nontrivial_option_processing")]
     fn report(&self) -> Self::Reporter<'_> {
         self.h2
@@ -397,16 +399,16 @@ where
     }
 
     #[cfg(not(feature = "nontrivial_option_processing"))]
-    type Reporter<'b>
-    where
-        Self: 'b,
-    = core::iter::Chain<
+    type Reporter<'b> = core::iter::Chain<
         core::iter::Map<H2::Reporter<'b>, fn(H2::Record<'b>) -> Self::Record<'b>>,
         core::iter::Map<
             core::iter::Zip<H1::Reporter<'b>, core::iter::Repeat<&'b [&'b str]>>,
             fn((H1::Record<'b>, &'b [&'b str])) -> Self::Record<'b>,
         >,
-    >;
+    >
+    where
+        Self: 'b,
+    ;
     #[cfg(not(feature = "nontrivial_option_processing"))]
     fn report(&self) -> Self::Reporter<'_> {
         fn first<'c, H1R, H2R>(
@@ -542,14 +544,14 @@ impl<T> wkc::Reporting for SimpleRendered<T>
 where
     T: SimpleRenderable,
 {
-    type Record<'a>
+    type Record<'a> = wkc::EmptyRecord
     where
         Self: 'a,
-    = wkc::EmptyRecord;
-    type Reporter<'a>
+    ;
+    type Reporter<'a> = core::iter::Once<wkc::EmptyRecord>
     where
         Self: 'a,
-    = core::iter::Once<wkc::EmptyRecord>;
+    ;
 
     fn report(&self) -> Self::Reporter<'_> {
         // Using a ConstantSliceRecord instead would be tempting, but that'd need a const return
@@ -601,14 +603,14 @@ use serde::Serialize;
 /// serde.
 ///
 /// A SimpleCBORHandler implementation can be turned into a [Handler] by wrapping it in
-/// [SimpleCBORWrapper::new].
+/// [SimpleWrapper::new].
+///
+/// The type a misnomer that can't be fixed until trait aliases are stabilized: It doesn't need to
+/// turn into CBOR, just something the Simple...Wrapper can (de)serialize.
 pub trait SimpleCBORHandler {
-    // Does it really make sense to constrain them? Seems so, as the only practical thing this
-    // trait is used for is later derivation, and that'd fail if any of those isn't satisified, so
-    // rather specifying them here for good error messages.
-    type Get: for<'de> serde::Serialize;
-    type Post: for<'de> serde::Deserialize<'de>;
-    type Put: for<'de> serde::Deserialize<'de>;
+    type Get;
+    type Post;
+    type Put;
 
     fn get(&mut self) -> Result<Self::Get, Code> {
         Err(code::METHOD_NOT_ALLOWED)
@@ -621,6 +623,25 @@ pub trait SimpleCBORHandler {
     }
 }
 
+/// Keeping them hidden to stay flexible; they don't need to be named for their'e either default or
+/// their users have aliases.
+mod sealed {
+    pub trait SimpleSerialization {
+        const CF: Option<u16>;
+    }
+
+    pub struct SerdeCBORSerialization;
+    pub struct MiniCBORSerialization;
+}
+use sealed::*;
+
+impl SimpleSerialization for SerdeCBORSerialization {
+    const CF: Option<u16> = Some(60);
+}
+impl SimpleSerialization for MiniCBORSerialization {
+    const CF: Option<u16> = Some(60);
+}
+
 /// Wrapper for resource handlers that are implemented in terms of GETting, POSTing or PUTting
 /// objects in CBOR format.
 ///
@@ -632,23 +653,31 @@ pub trait SimpleCBORHandler {
 /// aprameters into additional data available to the [SimpleCBORHandler]) can be built by
 /// forwarding to this (where any critical but already processed options would need to be masked
 /// from the message's option) or taking inspiration from it.
-pub struct SimpleCBORWrapper<H: SimpleCBORHandler> {
+pub struct SimpleWrapper<H, S: SimpleSerialization = SerdeCBORSerialization>
+where
+    H: SimpleCBORHandler,
+{
     handler: H,
+    _phantom: PhantomData<S>,
 }
+#[deprecated(note = "Renamed to SimpleWrapper")]
+pub type SimpleCBORWrapper<H> = SimpleWrapper<H, SerdeCBORSerialization>;
 
-impl<H: SimpleCBORHandler> SimpleCBORWrapper<H> {
-    pub fn new(handler: H) -> Self {
-        SimpleCBORWrapper { handler }
-    }
-
+impl<H, S> SimpleWrapper<H, S>
+where
+    H: SimpleCBORHandler,
+    S: SimpleSerialization,
+{
     fn check_get_options(request: &impl ReadableMessage) -> Result<Block2RequestData, Code> {
         let mut block2 = Ok(None);
 
         for o in request.options() {
             match o.number() {
                 option::ACCEPT => {
-                    if o.value_uint() != Some(60u8) {
-                        return Err(code::UNSUPPORTED_CONTENT_FORMAT);
+                    if let Some(cf) = S::CF {
+                        if o.value_uint() != Some(cf) {
+                            return Err(code::UNSUPPORTED_CONTENT_FORMAT);
+                        }
                     }
                 }
                 option::BLOCK2 => {
@@ -674,7 +703,12 @@ impl<H: SimpleCBORHandler> SimpleCBORWrapper<H> {
         for o in request.options() {
             match o.number() {
                 option::CONTENT_FORMAT => {
-                    if o.value_uint() != Some(60u8) {
+                    if let Some(cf) = S::CF {
+                        if o.value_uint() != Some(cf) {
+                            return Err(code::NOT_ACCEPTABLE);
+                        }
+                    } else {
+                        /* Nothing was configured, not letting the option pass */
                         return Err(code::NOT_ACCEPTABLE);
                     }
                 }
@@ -689,19 +723,56 @@ impl<H: SimpleCBORHandler> SimpleCBORWrapper<H> {
     }
 }
 
-pub enum SimpleCBORRequestData {
-    Get(Block2RequestData), // GET to be processed later, but all request opions were in order
-    Done(Code), // All done, just a response to emit -- if POST/PUT has been processed, or GET had a bad accept/option
-}
-use self::SimpleCBORRequestData::{Done, Get};
-impl<H> Handler for SimpleCBORWrapper<H>
+impl<H> SimpleWrapper<H, SerdeCBORSerialization>
 where
     H: SimpleCBORHandler,
-    H::Get: serde::Serialize,
+    H::Get: for<'de> serde::Serialize,
     H::Post: for<'de> serde::Deserialize<'de>,
     H::Put: for<'de> serde::Deserialize<'de>,
 {
-    type RequestData = SimpleCBORRequestData;
+    pub fn new(handler: H) -> Self {
+        SimpleWrapper {
+            handler,
+            _phantom: PhantomData,
+        }
+    }
+}
+
+#[cfg(feature = "minicbor")]
+impl<H> SimpleWrapper<H, MiniCBORSerialization>
+where
+    H: SimpleCBORHandler,
+    H::Get: for<'de> minicbor::Encode,
+    H::Post: for<'de> minicbor::Decode<'de>,
+    H::Put: for<'de> minicbor::Decode<'de>,
+{
+    pub fn new_minicbor(handler: H) -> Self {
+        SimpleWrapper {
+            handler,
+            _phantom: PhantomData,
+        }
+    }
+}
+
+/// Data carried around between a request and its response for [SimpleWrapper]s
+pub enum SimpleRequestData {
+    Get(Block2RequestData), // GET to be processed later, but all request opions were in order
+    Done(Code), // All done, just a response to emit -- if POST/PUT has been processed, or GET had a bad accept/option
+}
+#[deprecated(note = "Renamed to SimpleRequestData")]
+pub type SimpleCBORRequestData = SimpleRequestData;
+use self::SimpleRequestData::{Done, Get};
+
+// FIXME for all the below: deduplicate (but not sure how, without HKTs)
+
+impl<H> Handler for SimpleWrapper<H, SerdeCBORSerialization>
+where
+    H: SimpleCBORHandler,
+    H::Get: for<'de> serde::Serialize,
+    H::Post: for<'de> serde::Deserialize<'de>,
+    H::Put: for<'de> serde::Deserialize<'de>,
+{
+    type RequestData = SimpleRequestData;
 
     fn extract_request_data(&mut self, request: &impl ReadableMessage) -> Self::RequestData {
         match request.code().into() {
@@ -769,7 +840,94 @@ where
                             block2,
                             response,
                             |win| repr.serialize(&mut serde_cbor::ser::Serializer::new(win)),
-                            Some(60),
+                            SerdeCBORSerialization::CF,
+                        ) {
+                            Ok(()) => (),
+                            Err(_) => {
+                                // FIXME: Rewind all the written options
+                                response.set_code(codeconvert(code::INTERNAL_SERVER_ERROR));
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+}
+
+#[cfg(feature = "minicbor")]
+impl<H> Handler for SimpleWrapper<H, MiniCBORSerialization>
+where
+    H: SimpleCBORHandler,
+    H::Get: for<'de> minicbor::Encode,
+    H::Post: for<'de> minicbor::Decode<'de>,
+    H::Put: for<'de> minicbor::Decode<'de>,
+{
+    type RequestData = SimpleRequestData;
+
+    fn extract_request_data(&mut self, request: &impl ReadableMessage) -> Self::RequestData {
+        match request.code().into() {
+            code::GET => match Self::check_get_options(request) {
+                Err(e) => Done(e),
+                Ok(block) => Get(block),
+            },
+            code::POST => {
+                if let Err(e) = Self::check_postput_options(request) {
+                    return Done(e);
+                }
+
+                let parsed: Result<H::Post, _> = minicbor::decode(request.payload());
+                match parsed {
+                    Ok(p) => Done(self.handler.post(&p)),
+                    Err(_) => Done(code::BAD_REQUEST),
+                }
+            }
+            code::PUT => {
+                if let Err(e) = Self::check_postput_options(request) {
+                    return Done(e);
+                }
+
+                let parsed: Result<H::Put, _> = minicbor::decode(request.payload());
+                match parsed {
+                    Ok(p) => Done(self.handler.put(&p)),
+                    Err(_) => Done(code::BAD_REQUEST),
+                }
+            }
+            _ => Done(code::METHOD_NOT_ALLOWED),
+        }
+    }
+
+    fn estimate_length(&mut self, request: &Self::RequestData) -> usize {
+        match request {
+            Done(_) => 4,
+            Get(block) => (block.size() + 25).into(), // FIXME: hard-coded copied over from block2_write_with_cf's estimated overhead
+        }
+    }
+
+    fn build_response(
+        &mut self,
+        response: &mut impl MutableWritableMessage,
+        request: Self::RequestData,
+    ) {
+        match request {
+            Done(r) => {
+                response.set_code(codeconvert(r));
+                response.set_payload(b"");
+            }
+            Get(block2) => {
+                let repr = self.handler.get();
+                match repr {
+                    Err(e) => {
+                        response.set_code(codeconvert(e));
+                        response.set_payload(b"");
+                    }
+                    Ok(repr) => {
+                        response.set_code(codeconvert(code::CONTENT));
+                        match block2_write_with_cf(
+                            block2,
+                            response,
+                            |win| minicbor::encode(&repr, win),
+                            MiniCBORSerialization::CF,
                         ) {
                             Ok(()) => (),
                             Err(_) => {
