@@ -112,14 +112,14 @@ impl Record for EmptyRecord {
 }
 
 impl<H: Handler> Reporting for NotReporting<H> {
-    type Record<'a>
+    type Record<'a> = EmptyRecord
     where
         Self: 'a,
-    = EmptyRecord;
-    type Reporter<'a>
+    ;
+    type Reporter<'a> = core::iter::Empty<EmptyRecord>
     where
         Self: 'a,
-    = core::iter::Empty<EmptyRecord>;
+    ;
 
     fn report(&self) -> Self::Reporter<'_> {
         core::iter::empty()
@@ -202,14 +202,14 @@ impl<'a> Record for ConstantSliceRecord<'a> {
 }
 
 impl<'a, H: Handler> Reporting for ConstantSingleRecordReport<'a, H> {
-    type Record<'b>
+    type Record<'b> = ConstantSliceRecord<'b>
     where
         Self: 'b,
-    = ConstantSliceRecord<'b>;
-    type Reporter<'b>
+    ;
+    type Reporter<'b> = core::iter::Once<ConstantSliceRecord<'b>>
     where
         Self: 'b,
-    = core::iter::Once<ConstantSliceRecord<'b>>;
+    ;
 
     fn report(&self) -> Self::Reporter<'_> {
         core::iter::once(ConstantSliceRecord {
